﻿using System.Collections.Generic;

using DevExpress.XtraBars.Ribbon;
using DevExpress.XtraEditors;
using DevExpress.XtraEditors.Popup;

namespace TestProject
{
    /// <summary>
    /// 커스텀 토큰 에디터 드롭 다운 컨트롤
    /// </summary>
    public partial class CustomTokenEditDropDownControl : CustomTokenEditDropDownControlBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 토큰 리스트
        /// </summary>
        private List<TokenEditToken> tokenList = new List<TokenEditToken>();

        /// <summary>
        /// 클릭 항목
        /// </summary>
        private GalleryItem itemClicked;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CustomTokenEditDropDownControl()

        /// <summary>
        /// 생성자
        /// </summary>
        public CustomTokenEditDropDownControl()
        {
            InitializeComponent();

            this.customGalleryControl.Gallery.ShowItemText = true;

            this.customGalleryControl.Gallery.ItemClick += GalleryControlGallery_ItemClick;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public
        //////////////////////////////////////////////////////////////////////////////// Function

        #region 데이터 소스 설정하기 - SetDataSource(dataSource)

        /// <summary>
        /// 데이터 소스 설정하기
        /// </summary>
        /// <param name="dataSource">데이터 소스</param>
        public override void SetDataSource(object dataSource)
        {
            if(this.customGalleryControl.Gallery.Groups[0].Items.Count != 10)
            {
                CreateData();
            }
        }

        #endregion
        #region 항목 수 구하기 - GetItemCount()

        /// <summary>
        /// 항목 수 구하기
        /// </summary>
        /// <returns>항목 수</returns>
        public override int GetItemCount()
        {
            return this.tokenList.Count;
        }

        #endregion
        #region 폼 너비 계산하기 - CalcFormWidth()

        /// <summary>
        /// 폼 너비 계산하기
        /// </summary>
        /// <returns>폼 너비</returns>
        public override int CalcFormWidth()
        {
            return base.CalcFormWidth();
        }

        #endregion
        #region 폼 높이 계산하기 - CalcFormHeight(itemCount)

        /// <summary>
        /// 폼 높이 계산하기
        /// </summary>
        /// <param name="itemCount">항목 수</param>
        /// <returns>폼 높이</returns>
        public override int CalcFormHeight(int itemCount)
        {
            return 120;
        }

        #endregion
        #region 결과 값 구하기 - GetResultValue()

        /// <summary>
        /// 결과 값 구하기
        /// </summary>
        /// <returns>결과 값</returns>
        public override object GetResultValue()
        {
            int index = this.customGalleryControl.Gallery.Groups[0].Items.IndexOf(this.itemClicked);

            return this.tokenList[index];
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 갤러리 컨트롤 갤러리 항목 클릭시 처리하기 - GalleryControlGallery_ItemClick(sender, e)

        /// <summary>
        /// 갤러리 컨트롤 갤러리 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void GalleryControlGallery_ItemClick(object sender, GalleryItemClickEventArgs e)
        {
            this.itemClicked = e.Item;

            if((OwnerEdit as TokenEdit).EditValue != null && (OwnerEdit as TokenEdit).EditValue.ToString().Contains(e.Item.Caption))
            {
                return;
            }

            this.OwnerEdit.ClosePopup(PopupCloseMode.Normal);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 데이터 생성하기 - CreateData()

        /// <summary>
        /// 데이터 생성하기
        /// </summary>
        /// <returns>토큰 열거 가능형</returns>
        private IEnumerable<TokenEditToken> CreateData()
        {
            for(int i = 0; i < 10; i++)
            {
                this.tokenList.Add(new TokenEditToken("토큰 " + i.ToString(), "토큰 " + i.ToString()));

                this.customGalleryControl.Gallery.Groups[0].Items.Add
                (
                    new GalleryItem
                    (
                        this.imageCollection.Images[i],
                        this.tokenList[i].Description,
                        null
                    )
                );
            }

            return this.tokenList;
        }

        #endregion
    }
}